// $Id: image_composition.js,v 1.1.2.6 2007/12/09 08:22:43 clemenstolboom Exp $

/**
 * @file this script manages a binary tree of div's and img's
 *
 * each div is a node
 * each img is a leaf
 * the root is a div of class image-composition-container
 *
 * the root must contain at least one div or img
 * 
 * By walking up/down the tree images can be added
 *
 */
var sequence=0;
var currentEditObject = null;

$(document).ready(function() {
  $("#image-composition-ed//img.button").bind("click", function(){
    var splits = $(this).id().split('-');
    var cmd = splits[splits.length-1];
    executeCommand( cmd);
  });

  $("div.image-composition-container").each(function(){
    checkComposition( this);
    rebalance( $(this));
  });

  $("div.image-composition-container//img").each( function(){
    bindImage( $(this));
  });

  var nodeRef= $('#edit-node-ref-id').parent();
  nodeRef.append('<input id="edit-node-ref-apply" type="button" name="apply" value="Apply"/>');
  nodeRef.css( 'background-color', 'yellow');
  nodeRef.hide();
  
  $( "#edit-node-ref-apply").before( '<img id="edit-node-ref-image" src=""/>');

  $( "#edit-node-ref-apply").bind("click", function(){
    var nodeRef= getNodeRef();
    if( nodeRef == 0){
      $('#edit-node-ref-id').fadeOut( 'fast');
      $('#edit-node-ref-id').fadeIn( 'slow');

    } else {
      updateImage( $(currentEditObject), nodeRef);
    }
  });

});

function setNodeRef( obj, nodeRef){
  var id= 'i-c-' + nodeRef + '-' + getId();
  obj.id(id);
}

function getId(){
  return sequence++;
}

function updateImage( obj, id){
  if( id==0){
  } else {
    $.ajax( { 
      type: 'GET',
      url : '?q=image_composition/get_image_url_by_id/'+ id,
      success: function(data){
                 setNodeRef( obj, id);
                 $(obj).src( eval(data));
                 getStructure();
                 $('#edit-node-ref-image').src( $(obj).src());
               },
      error: function( xmlhttp){ alert( 'error ' + xmlhttp.status);}
    });
  }
}


function executeCommand( command){
  dir=  'h';
  switch( command){
    case 'hide':
      hideEditor();
      break;

    case 'top':
      dir= 'v';
      // fall through to left
    case 'left':
      currentEditObject.wrap( '<div class="' + dir + '"></div>');
      var obj =currentEditObject.before( getEmptyImage()).prev();
      var id= obj.id().match(/^i-c-(\d*)-\d*$/)[1];
      updateImage( obj, id);
      showEditor( bindImage( obj));
      break;

    case 'bottom':
      dir= 'v';
      // fall through to right
    case 'right':
      currentEditObject.wrap( '<div class="' + dir + '"></div>');
      var obj =currentEditObject.after( getEmptyImage()).next();
      var id= obj.id().match(/^i-c-(\d*)-\d*$/)[1];
      updateImage( obj, id);
      showEditor( bindImage( obj));
      break;

    case 'in':
    case 'out':
      zoom( command);
      break;
    case 'delete':
      objCurrent= $(currentEditObject);
      objParent= $(objCurrent).parent();
      objOther = null;

      if( objParent.is( '.image-composition-container')){
        // At the root. More then one node/leave(s)?
        if( objParent.children().length >1){
          // Ok to delete
          $(objCurrent).remove();
          objOther =  objParent.children()[0];
          showEditor( objOther);
        } else {
          // last image
          alert( "Cannot delete last item");
        }
      } else {
        // Not in the top so ok to delete
        $(objCurrent).remove();
        // unwrap from parent
        objOther = $(objParent).children()[0];
        $(objOther).insertAfter( $(objParent));
        objParent.remove();
        showEditor( objOther);
      }

      break;
  }
}

function zoom( command){
  if( command == 'in'){
    if( $(currentEditObject).children().length>0){
      currentEditObject= $(currentEditObject).children()[0];
    }
  } else if( command == 'out') {
    if( !$(currentEditObject).parent().is( '.image-composition-container')){
      currentEditObject= $(currentEditObject).parent();
    }
  }
  showEditor( currentEditObject);
}

function bindImage(img){
  img.bind('click', function(){
    showEditor( this);
  });
  return img;
}

function getNodeRef(){
  var val= 1 * $('#edit-node-ref-id').val();
  if( isNaN( val)){
    val = 0;
  }
  return val;
}

function getEmptyImage(){
  var seq = getId();
  var val= getNodeRef();

  id='i-c-' + val + '-' + seq;

  empty = $('#image-composition-empty').src();

  return '<img id="' + id 
  	+ '" src="' + empty + '"/>' + "\n";
}

/**
 * checks whether the workarea contains at least one image
 */
function checkComposition(obj){
  // using children('//img') breaks empty composition
  // using children('img') breaks non empty composition
  if( $(obj).find('img').size() == 0){
    // get rid off placeholder
    $(obj).empty();
    $(obj).append( getEmptyImage());
  }
}

function hideEditor(){
  $('#image-composition-ed').hide();
  $('#edit-node-ref-id').parent().hide();
}

/**
 * places the editor over the given object
 */
function showEditor( obj){
  var border= 20;

  currentEditObject= $(obj);

  $('#image-composition-ed').hide();
  if( currentEditObject != null){

    getStructure();
    rebalance( getParentContainer());

    // place editor partly around the current image
    var editor=  $('#image-composition-ed');
    editor.css( 'width', (2* border+ $(obj).width()) +'px'); 
    editor.css( 'height', (2* border+ $(obj).height()) + 'px'); 
    editor.css( 'top', ( $(obj).position()['top']- border) + 'px'); 
    editor.css( 'left', ( $(obj).position()['left']- border) + 'px'); 
    //editor.fadeIn("slow");
    editor.show();

    //$('#edit-node-ref-id').val( getNodeId( $(obj).id()));

    var nodeRef= $('#edit-node-ref-id').parent();
    nodeRef.css('display', 'float');
    nodeRef.css('position', 'absolute');
    nodeRef.css('z-index', '10');
    nodeRef.css('top', editor.position()['top'] + 'px');
    nodeRef.css('left', (editor.width() + editor.position()['left'])+ 'px');
    nodeRef.show();
  }
}

function getParentContainer(){
  if( currentEditObject != null){
    return $(currentEditObject).parents( '.image-composition-container')[0];
  }
  return null;
}

function rebalance( objParent){
  balanceStructure( objParent, 300, 300);
}

function balanceStructure( objRoot, width, height){
  objRoot = $(objRoot);
  objRoot.width( width + 'px');
  objRoot.height( height+ 'px');

  var numChild = $(objRoot).children().length;
  if( numChild>0){
    if( objRoot.is('.h')){
      width= Math.floor(width / numChild);
    } if( objRoot.is('.v')){
      height = Math.floor( height / numChild);
    }
  }

  $(objRoot).children().each( function(i){
    balanceStructure( this, width, height);
  });

}

function getStructure(){
  // get parent of current image / div
  var objParent = getParentContainer();
  $('#edit-composition-string').val(walkStructure( $(objParent).children()[0]));

}

/**
  * calculates the new structure of the given tree
  *
  * the calculation is based on class and children
  */
function walkStructure( objRoot){
  var result = "";

  objRoot=$( objRoot);

  var numChild= $( objRoot).children().length;
  if( numChild > 0){
    if( $(objRoot).is( '.h')){
      result="h";
    } else {
      result= "v";
    }
    result+="(";
    $(objRoot).children().each( function(i){
      if( i > 0){
        result+= ", ";
      }
      result += walkStructure( $(this));
    });

    result+= ")";
  } else {
    // get node id from 'i-c-(.*)-3'
    result = getNodeId($(objRoot).id());
  }
  return result;
}

function getNodeId( fullId){
  // get node id from 'i-c-(.*)-3'
  var result = fullId.replace( /^i\-c\-/, '');
  result = result.replace( /-[\d]+$/, '');
  return result;
}
