<?php

class hvlayout {
   var $_id = 0;
   var $_images = array();
   var $_width= 600;
   var $_padding= 3;
   var $_img_template= '<img alt="[alt]" src="[src]" id="[id]" width="[width]" height="[height]"/>';
   var $_composition = '';

   function hvlayout($width=600, $padding=3, $imagetemplate=''){
      $this->_width = $width;
      $this->_padding = $padding;

      if ($imagetemplate != ''){
         $this->_img_template = $imagetemplate;
      }
   }

   /**
    * calculates the layout based on an algorithme
    * 
    * the implementation of sort_images determines the algorithme
    */
   function do_layout(){

      while( count($this->_images) > 1){
        // how to merge this with a given composition ie h(1,v(2,3))?
        $this->sort_images();

        $this->do_layout_step();
      }

      if($this->_padding > 0){
        $path_to_module= drupal_get_path('module', 'image_composition') ;

        require_once( $path_to_module . '/math/Matrix.class.php');
        require_once( $path_to_module . '/math/GaussianElimination.class.php');

        $this->_equations = array();
        $this->_b = array();
        $this->_A = array();

        $this->add_borders( $this->_images[0]);

        // get all variable names
        $m_keys= array();
        foreach( $this->_A as $row){
          foreach( $row as $var => $value){
            $m_keys[ $var]++;
          }
        }

        $m = new matrix();
        $m->set_col_keys( array_keys( $m_keys));
        $m->set_row_keys( array_keys( $this->_A));


#print_r( $m_keys);
         $r=0;
         foreach( $this->_A as $in_row){
            foreach( $in_row as $var => $value){
              $m->set_value( $r, $var, $value);
            }
            $r++;
         }

$m->replace_vars( $this->_values);
$m->_fill_empty();
#$m->_print();

$b = new Matrix();
$b->set_row_keys( array(0));
$b->set_col_keys( array_keys( $this->_b));
$b->_A[0]=$this->_b;
$b->replace_vars( array( 'b'=>3));
#$b->transpose()->_print(false);

$g = new GaussianElimination();
$x = $g->solve( $m->_A, $b->_A[0]);
#print_r( $x);
#print_r( $this->_equations);

     }
   }

   /** 
    * add_borders
    *
    * to add borders we need to gather all equations
    * h(i_1,i_2) reads: horizontal layout of images 1 and 2
    * v(i_1,i_2) reads: vertical layout of images 1 and 2
    *
    * What is width and heigth of h(i_1,i_2) of v(i_1,i_2) 
    * when adding a border b (width or height)?
    *
    * known values are: w_1, h_1, w_2, h_2, b
    * unknown values are: s_1, s_2, w_total, h_total
    * ie: we can compose a equation based on the layout h or v
    *
    * DO NOT scale a composition ... that will scale the borders too.
    */
   function add_borders( &$image){
      if( $image['images']){
         // a composition
         $im_1 = $image['images'][0];
         $im_2 = $image['images'][1];

         $id_1 = $im_1['_id'];
         $id_2 = $im_2['_id'];
         $id_total = $image['_id'];

         $s_1 = 's_' . $id_1;
         $w_1 = 'w_' . $id_1;
         $h_1 = 'h_' . $id_1;

         if( $im_1['images']){
           // A composition is not supposed to scale
            $s_1 = '1';
         } else {
            $this->_values[ $w_1] = $im_1['width'];
            $this->_values[ $h_1] = $im_1['height'];
         }

         $s_2 = 's_' . $id_2;
         $w_2 = 'w_' . $id_2;
         $h_2 = 'h_' . $id_2;

         if( $im_2['images']){
            $s_2 = '1';
         } else {
            $this->_values[ $w_2] = $im_2['width'];
            $this->_values[ $h_2] = $im_2['height'];
         }

         $w_total = 'w_' . $id_total;
         $h_total = 'h_' . $id_total;

         if( $image['composition']=='h'){
            /*
             * Images next to each other:
             * w(1,2) : w_total == s_1 * w_1 + b + s_2 * w_2
             * h(1,2) : h_total == s_1 * h_1 == s_2 * h_2
             * ==>
             * s_1 * w_1 + s_2 * w_2 - 1 * w_total == b
             * s_1 * h_1 - s_2 * h_2               == 0
             * s_1 * h_1             - 1 * h_total == 0
             */
            $this->_equations[] = $s_1 . ' * ' . $w_1 . ' + ' . $s_2 . ' * ' . $w_2 . ' - 1 * ' . $w_total . ' = b';
            $row = array();
            $row[ ( $s_1== 1 ? $w_1: $s_1)] = ( $s_1 == 1 ? $s_1 : $w_1);
            $row[ ( $s_2== 1 ? $w_2: $s_2)] = ( $s_2 == 1 ? $s_2 : $w_2);
            $row[ $w_total] = -1;
            $this->_A[] = $row;
            $this->_b[] = b;

            $this->_equations[] = $s_1 . ' * ' . $h_1 . ' - ' . $s_2 . ' * ' . $h_2 .                        ' = 0';
            $row = array();
            $row[ ( $s_1== 1 ? $h_1: $s_1)] = ( $s_1 == 1 ? $s_1 : $h_1);
            $row[ ( $s_2== 1 ? $h_2: $s_2)] = '-' . ( $s_2 == 1 ? $s_2 : $h_2);
            $this->_A[] = $row;
            $this->_b[] = 0;

            $this->_equations[] = $s_1 . ' * ' . $h_1 .                               ' - 1 * ' . $h_total . ' = 0';
            $row = array();
            $row[ ( $s_1== 1 ? $h_1: $s_1)] = ( $s_1 == 1 ? $s_1 : $h_1);
            $row[ $h_total] = -1;
            $this->_A[] = $row;
            $this->_b[] = 0;
         } else {
            /*
             * w_v(1,2) : w_total == s_1 * w_1 == s_2 * w_2
             * h_v(1,2) : h_total == s_1 * h_1 + b + s_2 * h_2
             * ==>
             * s_1 * w_1 - s_2 * w_2               == 0
             * s_1 * h_1 + s_2 * h_2 - 1 * h_total == b
             * s_1 * w_1             - 1 * w_total == 0
             */
            $this->_equations[] = $s_1 . ' * ' . $w_1 . ' - ' . $s_2 . ' * ' . $w_2 .                        ' = 0';
            $row = array();
            $row[ ( $s_1== 1 ? $w_1: $s_1)] = ( $s_1 == 1 ? $s_1 : $w_1);
            $row[ ( $s_2== 1 ? $w_2: $s_2)] = '-' . ( $s_2 == 1 ? $s_2 : $w_2);
            $this->_A[] = $row;
            $this->_b[] = 0;

            $this->_equations[] = $s_1 . ' * ' . $h_1 . ' + ' . $s_2 . ' * ' . $h_2 . ' - 1 * ' . $h_total . ' = b';
            $row = array();
            $row[ ( $s_1== 1 ? $h_1: $s_1)] = ( $s_1 == 1 ? $s_1 : $h_1);
            $row[ ( $s_2== 1 ? $h_2: $s_2)] = ( $s_2 == 1 ? $s_2 : $h_2);
            $row[ $h_total] = -1;
            $this->_A[] = $row;
            $this->_b[] = b;

            $this->_equations[] = $s_1 . ' * ' . $w_1 .                               ' - 1 * ' . $w_total . ' = 0';
            $row = array();
            $row[ ( $s_1== 1 ? $w_1: $s_1)] = ( $s_1 == 1 ? $s_1 : $w_1);
            $row[ $w_total] = -1;
            $this->_A[] = $row;
            $this->_b[] = 0;
         }
         $this->add_borders($im_1);
         $this->add_borders($im_2);
      }
   }

  /**
   * extract the id's from the composition
   *
   * h(v(1,2),3) extract 1,2,3
   * h(v(1,1),1) extract 1
   *
   * @return array of image id's
   */
  function get_ids(){
    return array_values( $this->get_leafs( $this->get_structure()));
  } 
    
  /*
   * extract the leaf values from a tree
   */
  function get_leafs( $structure){
    if( is_array( $structure)){
      $result= array_merge( $this->get_leafs( $structure[0]), $this->get_leafs( $structure[1])); 
    } 
    else {
      $result= array( "_" . $structure => $structure);
    } 
    return $result;
  } 

  /*
   * sets the structure
   */
  function set_composition( $composition){
    $this->_composition = $composition;
  }
  
  /**
   * parses a composition string into a tree structure
   *
   * composition v(h(1,2),3) is split
   * into tree [ d=>'v', [ 'd'=> h [ 1,2]], 3]
   *
   * TODO: this is a lousy parser...if the syntax is changed
   *       a little this collapses. Next implementation 
   *       should to this recusively.
   */
  function get_structure(){
    $c= str_replace( ' ', '', $this->_composition);
  
    // Tokenize composition
    $tokens = preg_split( '/[\(\),]/', $c);
   
    $a= array();
   
    foreach( $tokens as $token){
      if( $token=='h' || $token=='v'){
        array_push( $a, array( 'd'=> $token));
      }
      else if( $token== ''){ 
        $b= array_pop($a);
        $top = array_pop($a);
        $top[] =$b;
        array_push($a, $top);
      }
      else {
        $top = array_pop($a);
        $top[] =$token;
        array_push($a, $top);
      } 
    } 
  
    // this is nasty ... why do i need this?
    // OK : array.push(array(0) === root)
    // return root
    return $a[0][0];
  }

  function get_html(){
    $width= $this->_width;

    if( $this->_composition[0]=='h' || $this->_composition[0]=='v'){
      $structure = $this->get_structure();
      // TODO: Now transform the structure to an image tree
      $im = $this->restructure( $structure);
    }
    else {
      $this->do_layout();
      $im = $this->_images[ 0];
    }

    return $im== null ? '' : $this->to_html_composition($im, $width, 0);
  }

  function to_html_composition(&$im, $width, $top= 0, $left= 0){
    $result='';

    $ratio= $im["ratio"];

    if( $im["composition"]){
      if( $im["composition"] == 'v'){
        $height= ($im["images"][0]["ratio"]) * $width;

        $result.= '<div class="v" style="float:left;width:'. $width .'px">';
        $result.= $this->to_html_composition( $im["images"][0],
            $width, $top, $left);

        $result.= $this->to_html_composition( $im["images"][1],
            $width, $top + $height, $left);
        $result.= "</div>";

      } else {
        // horizontal
        $height= $ratio * $width;
        $w1= $height / $im["images"][0]["ratio"] ;
        $result.= '<div class="h" style="float:left;">';

        $result.= $this->to_html_composition( $im["images"][0],
            $w1, $top, $left);

        $result.= $this->to_html_composition( $im["images"][1],
            $width-$w1, $top, $left+ $w1);
        $result.= "</div>";
      }

    } else {
      $height= $ratio * $width;

      $result .= "\n" .
        str_replace('[url]',$im['url'],
          str_replace('[src]',$im['src'],
            str_replace('[width]', floor($width),
              str_replace('[height]', floor($height),
                str_replace('[id]', 'i-c-' . $im['noderef'],
                  str_replace('[alt]',$im['alt'],
                    $this->_img_template
                  )
                )
              )
            )
          )
        );
    }

    return $result;
  }

  function do_layout_step(){

    $im1 = array_pop( $this->_images);
    $w1 = $im1['width'];
    $h1 = $im1['height'];
    $t1 = $im1['type'];
    $a1 = $im1["ratio"];

    $im2 = array_pop( $this->_images);
    $w2 = $im2['width'];
    $h2 = $im2['height'];
    $t2 = $im2['type'];

    $sequence = "$t1$t2";

    if( $t1=='P' ){
      // horizontal has preference
      $composition = "h";

      $h = min( $h1, $h2);
      $w = ($w1 / $h1 + $w2 / $h2) * $h;
      $q = $h * $h;
      $ratio = $h/$w;

      $im1["_width"] = $h / $im1["ratio"];
      $im1["_height"] = $h;
      $im2["_width"] = $h / $im2["ratio"];
      $im2["_height"] = $h;

     } else {
      $composition = "v";

      $w = min( $w1, $w2);
      $h = ( $h1 / $w1 + $h2 / $w2) * $w;
      $q = $w * $h;
      $ratio = $h/$w;

      $im1["_width"] = $w;
      $im1["_height"] = $w * $im1["ratio"];
      $im2["_width"] = $w;
      $im2["_height"] = $w * $im2["ratio"];

    }

    $type = $h>$w ? 'P' : 'L';

    array_push( $this->_images
      , array( 'ratio'=> $ratio
          , 'width' => $w
          , 'height'=> $h
          , 'quality' => $q
          , 'composition'=> $composition
          , '_id' => $this->get_id()
          , 'type' => $type
          , 'images' => array($im1, $im2)
        )
    );
  } 

  /**
   * appends an image to the list of images
   *
   * invalid images are failed silently
   *
   */
  function add_image($image){
    $filename = $image["filename"];

    list($w, $h) = getimagesize($filename);
    if($w==0 || $h==0){
      return;
    }

    $ratio = $h/$w;

    $this->_images[] = array(
      'filename' => $filename,
      'width'=>$w,
      'url'=> $image['url'],
      'src'=> $image['src'],
      'alt'=> $image['alt'],
      'height'=>$h,
      'quality' => ($w*$h),
      'ratio' => $ratio,
      'type'=> ($ratio<1? 'L':'P'),
      'noderef'=> $image[ 'noderef'],
      'node_id'=> $image[ 'node_id'],
      '_id' => $this->get_id(),
    );
  }

  function get_image_by_ref($noderef){
    static $index= 0;

    if($noderef== '%'){
      return $this->_images[$index++];
    }
    foreach( $this->_images as $image){
      if( $image['node_id']== $noderef){
        return $image;
      }
    }
  } 

  function sort_images(){
    foreach ($this->_images as $key => $row) {
       $quality[$key]  = $row['quality'];
    }

    // sort on quality then on type
    array_multisort( $quality, SORT_DESC, $this->_images);
  }

  function get_id(){
     return $this->_id++;
  }

  function restructure( $tree){
    if( is_array($tree) ){
      // it's a node so recurse
      $im0= $this->restructure($tree['0']);
      $im1= $this->restructure($tree['1']);

      $result= array( 
        'composition' => $tree['d'],
      );

      // Make this widht height agnostic
      if( $result['composition']== 'h'){
        // 1/ratio = 1/r1 + 1/r2
        $temp = 1/$im0['ratio'] + 1/$im1['ratio'];
        $ratio= 1/$temp;
      }
      else {
        // ratio = r1 + r2
        $ratio = $im0['ratio'] + $im1['ratio'];
      }
      $result[ 'ratio']= $ratio;
      $result[ 'images']= array( $im0, $im1);

      return $result;
    }
    else {
      // it's a leaf
      return $this->get_image_by_ref($tree);
    }
  }
}
